@@ -5,9 +5,7 @@ import getpass
 import inspect
 import os
 import shutil
-import socket
 import tempfile
-import traceback
 from contextlib import contextmanager, suppress
 from pathlib import Path
 
@@ -19,12 +17,21 @@ from pytest_mproc.fixtures import Node, Global
 
 import pytest
 import _pytest.terminal
+import socket
 
 from multiprocessing import cpu_count
 from pytest_mproc.coordinator import Coordinator
 from pytest_mproc.utils import is_degraded
 
 
+def _get_ip_addr():
+    try:
+        return socket.gethostbyname(socket.gethostname())
+    except Exception:
+        print(">>> Cannot get ip address of host.  Return 127.0.0.1 (localhost)")
+        return "127.0.0.1"
+
+
 def parse_numprocesses(s):
     """
     A little bit of processing to get number of parallel processes to use (since "auto" can be used to represent
@@ -95,6 +102,8 @@ def pytest_cmdline_main(config):
 
     Mostly taken from other implementations (such as xdist)
     """
+    if config.option.collectonly:
+        return
     worker = getattr(config.option, "mproc_worker", None)
     mproc_server_port = getattr(config.option, 'mproc_server_port', None)
     mproc_client_connect = getattr(config.option, "mproc_client_connect", None)
@@ -111,7 +120,7 @@ def pytest_cmdline_main(config):
                             (mproc_server_port is None and mproc_client_connect is None)) else \
         RoleEnum.COORDINATOR
     if role == RoleEnum.MASTER:
-        print(f"Running as {role}: {socket.gethostbyname(socket.gethostname())}:{mproc_server_port}")
+        print(f"Running as {role}: {_get_ip_addr()}:{mproc_server_port}")
 
     # validation
     if getattr(config.option, "mproc_numcores", None) is None or is_degraded() or getattr(config.option, "mproc_disabled"):
@@ -130,7 +139,7 @@ def pytest_cmdline_main(config):
     elif mproc_server_port:
         if config.option.numprocesses < 0:
             raise pytest.UsageError("Number of cores must be greater than or equal to zero when running as a master")
-        mpconfig = MPManagerConfig(global_mgr_host=socket.gethostbyname(socket.gethostname()),
+        mpconfig = MPManagerConfig(global_mgr_host=_get_ip_addr(),
                                    global_mgr_port=mproc_server_port,
                                    role=role,
                                    num_processes=config.option.mproc_numcores)
@@ -179,6 +188,13 @@ def pytest_cmdline_main(config):
     config.mproc_node_manager = Node.Manager(mpconfig)
 
 
+def pytest_sessionstart(session):
+    with suppress(Exception):
+        reporter = session.config.pluginmanager.getplugin('terminalreporter')
+        if reporter:
+                reporter._session = session
+
+
 @pytest.mark.trylast
 def pytest_configure(config):
     if getattr(config.option, "mproc_numcores", None) is None or is_degraded() or getattr(config.option, "mproc_disabled"):
@@ -189,8 +205,6 @@ def pytest_configure(config):
 
 @pytest.mark.tryfirst
 def pytest_fixture_setup(fixturedef, request):
-    if not getattr(request.config.option, "mproc_numcores") or fixturedef.scope not in ['node', 'global']:
-        return _pytest.fixtures.pytest_fixture_setup(fixturedef, request)
     my_cache_key = fixturedef.cache_key(request)
     if request.config.option.mpconfig.role == RoleEnum.WORKER:
         if request.config.option.mpconfig.global_fixtures is None:
@@ -199,7 +213,27 @@ def pytest_fixture_setup(fixturedef, request):
         if request.config.option.mpconfig.node_fixtures is None:
             client = Node.Manager(request.config.option.mpconfig, force_as_client=True)
             request.config.option.mpconfig.node_fixtures = client.get_fixtures()
+        if fixturedef.scope == 'global':
+            fixturedef.cached_result = (request.config.option.mpconfig.global_fixtures.get(fixturedef.argname),
+                                        my_cache_key,
+                                        None)
+            return request.config.option.mpconfig.global_fixtures.get(fixturedef.argname)
+        elif fixturedef.scope == 'node':
+            fixturedef.cached_result = (request.config.option.mpconfig.node_fixtures.get(fixturedef.argname),
+                                        my_cache_key,
+                                        None)
+            return request.config.option.mpconfig.node_fixtures.get(fixturedef.argname)
+        else:
+            return
 
+    if not getattr(request.config.option, "mproc_numcores") or fixturedef.scope not in ['node', 'global']:
+        return
+    if not hasattr(request.config, "generated_fixtures"):
+        request.config.generated_fixtures = []
+    if request.config.option.mpconfig.global_fixtures is None:
+        request.config.option.mpconfig.global_fixtures = {}
+    if request.config.option.mpconfig.node_fixtures is None:
+        request.config.option.mpconfig.node_fixtures = {}
     if fixturedef.scope == 'node':
         assert request.config.option.mpconfig.node_fixtures.get(fixturedef.argname) is not None
         fixturedef.cached_result = (request.config.option.mpconfig.node_fixtures.get(fixturedef.argname),
@@ -217,56 +251,56 @@ def pytest_fixture_setup(fixturedef, request):
         # return request.config.mproc_global_manager.get_fixturedef(fixturedef, request)
 
 
-def process_fixturedef(name: str, item, session, global_fixtures, node_fixtures):
-    generated = []
-    try:
-        mpconfig: MPManagerConfig = session.config.option.mpconfig
-        assert mpconfig.role != RoleEnum.WORKER
-        request = item._request
-        fixturedef = item._fixtureinfo.name2fixturedefs.get(name, None)
-        if fixturedef:
-            for argname in fixturedef[0].argnames:
-                generated += process_fixturedef(argname, item, session, global_fixtures, node_fixtures)
-        if not fixturedef or fixturedef[0].scope not in ['global', 'node']:
-            return generated
-        if mpconfig.role == RoleEnum.MASTER and fixturedef and fixturedef[0].scope == 'global' and name not in global_fixtures:
-            global_fixtures[name] = _pytest.fixtures.resolve_fixture_function(fixturedef[0], request)
-            val = global_fixtures[name](*fixturedef[0].argnames)
-            if inspect.isgenerator(val):
-                generated.append(val)
-                val = next(val)
-            mpconfig.global_fixtures[name] = val
-        if mpconfig.role in [RoleEnum.MASTER, RoleEnum.COORDINATOR] and fixturedef and \
-                fixturedef[0].scope == 'node' and name not in node_fixtures:
-            node_fixtures[name] = _pytest.fixtures.resolve_fixture_function(fixturedef[0], request)
-            val = node_fixtures[name](*fixturedef[0].argnames)
-            if inspect.isgenerator(val):
-                generated.append(val)
-                val = next(val)
-            mpconfig.node_fixtures[name] = val
-
-        return generated
-    except Exception as e:
-        print(traceback.format_exc())
-        session.shouldfail = str(e)
-        return generated
+def process_fixturedef(fixturedef, request):
+    generated = request.config.generated_fixtures
+    global_fixtures = request.config.option.mpconfig.global_fixtures
+    node_fixtures = request.config.option.mpconfig.node_fixtures
+    config = request.config
+    mpconfig: MPManagerConfig = config.option.mpconfig
+    if not fixturedef or fixturedef.scope not in ['global', 'node']:
+        return
+    name = fixturedef.argname
+    assert mpconfig.role != RoleEnum.WORKER
+    if mpconfig.role == RoleEnum.MASTER and fixturedef.scope == 'global' and name not in global_fixtures:
+        global_fixtures[name] = _pytest.fixtures.resolve_fixture_function(fixturedef, request)
+        val = global_fixtures[name](*fixturedef.argnames)
+        if inspect.isgenerator(val):
+            generated.append(val)
+            val = next(val)
+        mpconfig.global_fixtures[name] = val
+    if mpconfig.role in [RoleEnum.MASTER, RoleEnum.COORDINATOR] and fixturedef.scope == 'node'\
+            and name not in node_fixtures:
+        node_fixtures[name] = _pytest.fixtures.resolve_fixture_function(fixturedef, request)
+        val = node_fixtures[name](*fixturedef.argnames)
+        if inspect.isgenerator(val):
+            generated.append(val)
+            val = next(val)
+        mpconfig.node_fixtures[name] = val
 
 
-@pytest.mark.tryfirst
 def pytest_runtestloop(session):
+    if session.config.option.collectonly:
+        return
+    reporter = session.config.pluginmanager.getplugin('terminalreporter')
+    if reporter:
+        reporter.tests_count = len(session.items)
     if session.testsfailed and not session.config.option.continue_on_collection_errors:
         raise session.Interrupted("%d errors during collection" % session.testsfailed)
 
     if session.config.option.collectonly:
+        if session.config.option.mpconfig.role in [RoleEnum.MASTER, RoleEnum.COORDINATOR]:
+            if hasattr(session.config, "coordinator"):
+                session.config.coordinator.kill()
         return  # should never really get here, but for consistency
-
+    if len(session.items) == 0:
+        return
     if getattr(session.config.option, "mproc_numcores", None) is None or is_degraded() or getattr(session.config.option, "mproc_disabled"):
         # return of None indicates other hook impls will be executed to do the task at hand
         # aka, let the basic hook handle it from here, no distributed processing to be done
         return
 
     mpconfig = session.config.option.mpconfig
-    session.generated_fixtures = []
+    session.config.generated_fixtures = []
     if mpconfig.role != RoleEnum.WORKER:
         if mpconfig.global_fixtures is None:
             mpconfig.global_fixtures = {}
@@ -277,21 +311,38 @@ def pytest_runtestloop(session):
                 break
             if hasattr(item, "_request"):
                 for name in item._fixtureinfo.argnames:
-                    if name not in mpconfig.global_fixtures and name not in mpconfig.node_fixtures:
-                        session.generated_fixtures += process_fixturedef(name, item, session,
-                                                                         mpconfig.global_fixtures,
-                                                                         mpconfig.node_fixtures)
-                    if session.shouldfail:
+                    try:
+                        if name not in mpconfig.global_fixtures and name not in mpconfig.node_fixtures:
+                            fixturedef = item._fixtureinfo.name2fixturedefs.get(name, [None])[0]
+                            if not fixturedef:
+                                continue
+                            for argname in fixturedef.argnames:
+                                fixdef = item._fixtureinfo.name2fixturedefs.get(argname, [None])[0]
+                                process_fixturedef(fixdef, item._request)
+
+                            process_fixturedef(fixturedef, item._request)
+                    except Exception as e:
+                        import traceback
+                        session.shouldfail = f"Exception in fixture: {e.__class__.__name__} raised with msg '{str(e)}'"
+                        print(f">>> Fixture ERROR: {traceback.format_exc()}")
                         break
-        if mpconfig.role == RoleEnum.MASTER:
-            session.config.mproc_global_manager.register_fixtures(mpconfig.global_fixtures)
-        session.config.mproc_node_manager.register_fixtures(mpconfig.node_fixtures)
+
     if session.shouldfail:
-        print(f">>> Fixture ERROR: {session.shouldfail}")
         if mpconfig.role in [RoleEnum.MASTER, RoleEnum.COORDINATOR]:
             if hasattr(session.config, "coordinator"):
                 session.config.coordinator.kill()
         raise session.Failed(session.shouldfail)
+    elif mpconfig.role != RoleEnum.WORKER:
+        try:
+            if mpconfig.role == RoleEnum.MASTER:
+                session.config.mproc_global_manager.register_fixtures(mpconfig.global_fixtures)
+            session.config.mproc_node_manager.register_fixtures(mpconfig.node_fixtures)
+        except Exception as e:
+            session.shouldfail = f"Exception in fixture: {e.__class__.__name__} raised with msg '{str(e)}'"
+            print(f">>>> Error registering node/global level fixture(s): {str(e)}")
+            if hasattr(session.config, "coordinator"):
+                session.config.coordinator.kill()
+            raise session.Failed(session.shouldfail)
 
     if mpconfig.role != RoleEnum.WORKER:
         if not session.config.getvalue("collectonly"):
@@ -313,6 +364,8 @@ def pytest_runtestloop(session):
 
 
 def pytest_terminal_summary(terminalreporter: _pytest.terminal.TerminalReporter, exitstatus, config):
+    if config.option.collectonly:
+        return
     if config.option.mpconfig.role != RoleEnum.MASTER:
         terminalreporter.line(">>>>>>>>>> This is a satellite processing node. "
                               "Please see master node output for actual test summary <<<<<<<<<<<<",
@@ -320,6 +373,8 @@ def pytest_terminal_summary(terminalreporter: _pytest.terminal.TerminalReporter,
 
 
 def pytest_sessionfinish(session):
+    if session.config.option.collectonly:
+        return
     mpconfig = session.config.option.mpconfig
     if mpconfig.role == RoleEnum.MASTER:
         try:
@@ -331,13 +386,21 @@ def pytest_sessionfinish(session):
             session.config.mproc_node_manager.shutdown()
         except Exception as e:
             print(">>> INTERNAL Error shutting down mproc manager")
-    generated = getattr(session, "generated_fixtures", [])
+    generated = getattr(session.config, "generated_fixtures", [])
+    errors = []
     for item in generated:
         try:
-            next(item)
-            raise Exception(f"Fixture generator did not stop {item}")
-        except StopIteration:
-            pass
+            try:
+                next(item)
+                raise Exception(f"Fixture generator did not stop {item}")
+            except StopIteration:
+                pass
+        except Exception as e:
+            print(f">>> Error in exit of fixture context {e}")
+            errors.append(e)
+    if errors:
+        raise pytest.UsageError(">>> One or more global or node-level fixtures threw an Exception on exit from context") \
+            from errors[0]
 
 ################
 # Process-safe temp dir
